@@ -41,13 +41,14 @@ static const char *filename=NULL; /* current file name. Used for printing
 static int file_nr=0;
 
 
-static int flag_mask; /* mask of currently set flags */
+static unsigned int flag_mask; /* mask of currently set flags */
 
 /* devices */
-static int cur_devs; /* current number of defined devices */
+static unsigned int cur_devs; /* current number of defined devices */
 static int cur_dev; /* device being filled in. If negative, none */
 static int trusted=0; /* is the currently parsed device entry trusted? */
-static int nr_dev; /* number of devices that the current table can hold */
+static unsigned int nr_dev; /* number of devices that the current table can 
+			       hold */
 struct device *devices; /* the device table */
 static int token_nr; /* number of tokens in line */
 
@@ -63,6 +64,7 @@ unsigned int mtools_no_vfat=0;
 unsigned int mtools_numeric_tail=1;
 unsigned int mtools_dotted_dir=0;
 unsigned int mtools_twenty_four_hour_clock=1;
+unsigned int mtools_lock_timeout=30;
 unsigned int mtools_default_codepage=850;
 const char *mtools_date_string="yyyy-mm-dd";
 char *country_string=0;
@@ -90,12 +92,13 @@ static switches_t global_switches[] = {
       (caddr_t) &mtools_twenty_four_hour_clock, T_UINT },
     { "MTOOLS_DATE_STRING",
       (caddr_t) &mtools_date_string, T_STRING },
+    { "MTOOLS_LOCK_TIMEOUT", (caddr_t) &mtools_lock_timeout, T_UINT },
     { "DEFAULT_CODEPAGE", (caddr_t) &mtools_default_codepage, T_UINT }
 };
 
 typedef struct {
     const char *name;
-    int flag;
+    unsigned int flag;
 } flags_t;
 
 static flags_t openflags[] = {
@@ -129,7 +132,7 @@ static flags_t misc_flags[] = {
 static struct {
     const char *name;
     signed char fat_bits;
-    int tracks;
+    unsigned int tracks;
     unsigned short heads;
     unsigned short sectors;
 } default_formats[] = {
@@ -173,6 +176,12 @@ static switches_t dswitches[]= {
     { "CODEPAGE", OFFS(codepage), T_UINT }
 };
 
+static __inline__ char ch_toupper(char ch)
+{
+    return (char) toupper( (unsigned char) ch);
+}
+
+
 static void maintain_default_drive(char drive)
 {
     if(default_drive == ':')
@@ -190,6 +199,7 @@ char get_default_drive(void)
 	return 'A';
 }
 
+static void syntax(const char *msg, int thisLine) NORETURN;
 static void syntax(const char *msg, int thisLine)
 {
     char drive='\0';
@@ -215,7 +225,8 @@ static void get_env_conf(void)
 	    if(global_switches[i].type == T_INT)
 		* ((int *)global_switches[i].address) = (int) strtol(s,0,0);
 	    if(global_switches[i].type == T_UINT)
-		* ((int *)global_switches[i].address) = (unsigned int) strtoul(s,0,0);
+		* ((unsigned int *)global_switches[i].address) =
+			(unsigned int) strtoul(s,0,0);
 	    else if (global_switches[i].type == T_STRING)
 		* ((char **)global_switches[i].address) = s;
 	}
@@ -316,7 +327,7 @@ static unsigned int get_unumber(void)
     return n;
 }
 
-static unsigned int get_number(void)
+static int get_number(void)
 {
     char *last;
     int n;
@@ -334,9 +345,9 @@ static unsigned int get_number(void)
 /* purge all entries pertaining to a given drive from the table */
 static void purge(char drive, int fn)
 {
-    int i,j;
+    unsigned int i, j;
 
-    drive = toupper(drive);
+    drive = ch_toupper(drive);
     for(j=0, i=0; i < cur_devs; i++) {
 	if(devices[i].drive != drive ||
 	   devices[i].file_nr == fn)
@@ -366,7 +377,7 @@ static void init_drive(void)
 /* prepends a device to the table */
 static void prepend(void)
 {
-    int i;
+    unsigned int i;
 
     grow();
     for(i=cur_devs; i>0; i--)
@@ -389,12 +400,10 @@ static void append(void)
 
 static void finish_drive_clause(void)
 {
-    char drive;
     if(cur_dev == -1) {
 	trusted = 0;
 	return;
     }
-    drive = devices[cur_dev].drive;
     if(!devices[cur_dev].name)
 	syntax("missing filename", 0);
     if(devices[cur_dev].tracks ||
@@ -430,7 +439,7 @@ static int set_var(struct switches_l *switches, int nr,
 	if(match_token(switches[i].name)) {
 	    expect_char('=');
 	    if(switches[i].type == T_UINT)
-		* ((int *)((long)switches[i].address+base_address)) =
+		* ((unsigned int *)((long)switches[i].address+base_address)) =
 		    get_unumber();
 	    if(switches[i].type == T_INT)
 		* ((int *)((long)switches[i].address+base_address)) =
@@ -507,30 +516,27 @@ static int set_def_format(struct device *dev)
 
 static int parse_one(int privilege);
 
-/* check for offset embedded in file name, in the form file@@offset[SKMG] */
-static off_t get_offset(char *name) {
+void set_cmd_line_image(char *img) {
   char *ofsp;
-  off_t ofs;
 
-  ofsp = strstr(devices[cur_dev].name, "@@");
-  if (ofsp == NULL)
-    return 0; /* no separator */
-  ofs = str_to_offset(ofsp+2);
-  *ofsp = '\0';                              /* truncate file name */
-  return ofs;
-}
-
-void set_cmd_line_image(char *img, int flags) {
-  char *name;
   prepend();
   devices[cur_dev].drive = ':';
   default_drive = ':';
-  devices[cur_dev].name = name = strdup(img);
+
+  ofsp = strstr(img, "@@");
+  if (ofsp == NULL) {
+    /* no separator => no offset */
+    devices[cur_dev].name = strdup(img);
+    devices[cur_dev].offset = 0;
+  } else {
+    devices[cur_dev].name = strndup(img, ofsp - img);
+    devices[cur_dev].offset = str_to_offset(ofsp+2);
+  }
+
   devices[cur_dev].fat_bits = 0;
   devices[cur_dev].tracks = 0;
   devices[cur_dev].heads = 0;
   devices[cur_dev].sectors = 0;
-  devices[cur_dev].offset = get_offset(name);
   if (strchr(devices[cur_dev].name, '|')) {
     char *pipechar = strchr(devices[cur_dev].name, '|');
     *pipechar = 0;
@@ -560,7 +566,7 @@ static void parse_old_device_line(char drive)
 	
     /* reserve slot */
     append();
-    items = sscanf(token,"%c %s %i %i %i %i %li",
+    items = sscanf(token,"%c %s %i %ui %ui %ui %li",
 		   &devices[cur_dev].drive,name,&devices[cur_dev].fat_bits,
 		   &devices[cur_dev].tracks,&devices[cur_dev].heads,
 		   &devices[cur_dev].sectors, &offset);
@@ -584,14 +590,13 @@ static void parse_old_device_line(char drive)
 	case 4:
 	case 5:
 	    syntax("bad number of parameters", 1);
-	    exit(1);
     }
     if(!devices[cur_dev].tracks){
 	devices[cur_dev].sectors = 0;
 	devices[cur_dev].heads = 0;
     }
 	
-    devices[cur_dev].drive = toupper(devices[cur_dev].drive);
+    devices[cur_dev].drive = ch_toupper(devices[cur_dev].drive);
     maintain_default_drive(devices[cur_dev].drive);
     if (!(devices[cur_dev].name = strdup(name))) {
 	printOom();
@@ -632,7 +637,7 @@ static int parse_one(int privilege)
 	memset((char*)(devices+cur_dev), 0, sizeof(*devices));
 	trusted = privilege;
 	flag_mask = 0;
-	devices[cur_dev].drive = toupper(token[0]);
+	devices[cur_dev].drive = ch_toupper(token[0]);
 	maintain_default_drive(devices[cur_dev].drive);
 	expect_char(':');
 	return 1;
@@ -658,7 +663,7 @@ static int parse_one(int privilege)
 static int parse(const char *name, int privilege)
 {
     if(fp) {
-	fprintf(stderr, "File descriptor already set (%p)!\n", fp);
+	fprintf(stderr, "File descriptor already set!\n");
 	exit(1);
     }
     fp = fopen(name, "r");
@@ -733,14 +738,15 @@ void read_config(void)
 	mtools_fat_compatibility=1;
 }
 
-void mtoolstest(int argc, char **argv, int type)
+void mtoolstest(int argc, char **argv, int type  UNUSEDP) NORETURN;
+void mtoolstest(int argc, char **argv, int type  UNUSEDP)
 {
     /* testing purposes only */
     struct device *dev;
     char drive='\0';
 
     if(argc > 1 && argv[1][0] && argv[1][1] == ':') {
-	drive = toupper(argv[1][0]);
+	drive = ch_toupper(argv[1][0]);
     }
 
     for (dev=devices; dev->name; dev++) {
@@ -809,3 +815,9 @@ void mtoolstest(int argc, char **argv, int type)
 
     exit(0);
 }
+
+/*
+ * Local Variables:
+ * c-basic-offset: 4
+ * End:
+ */
